@@ -19,9 +19,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 #
-import time
-
-from tests.compat import mock, unittest
+from tests.compat import unittest
 from tests.unit import AWSMockServiceTestCase
 from tests.unit import MockServiceWithConfigTestCase
 
@@ -50,6 +48,31 @@ class TestSignatureAlteration(AWSMockServiceTestCase):
         )
 
 
+class TestPresigned(MockServiceWithConfigTestCase):
+    connection_class = S3Connection
+
+    def test_presign_respect_query_auth(self):
+        self.config = {
+            's3': {
+                'use-sigv4': False,
+            }
+        }
+
+        conn = self.connection_class(
+            aws_access_key_id='less',
+            aws_secret_access_key='more',
+            host='s3.amazonaws.com'
+        )
+
+        url_enabled = conn.generate_url(86400, 'GET', bucket='examplebucket',
+                                        key='test.txt', query_auth=True)
+
+        url_disabled = conn.generate_url(86400, 'GET', bucket='examplebucket',
+                                         key='test.txt', query_auth=False)
+        self.assertIn('Signature=', url_enabled)
+        self.assertNotIn('Signature=', url_disabled)
+
+
 class TestSigV4HostError(MockServiceWithConfigTestCase):
     connection_class = S3Connection
 
@@ -111,9 +134,12 @@ class TestSigV4Presigned(MockServiceWithConfigTestCase):
         # Here we force an input iso_date to ensure we always get the
         # same signature.
         url = conn.generate_url_sigv4(86400, 'GET', bucket='examplebucket',
-            key='test.txt', iso_date='20140625T000000Z')
+                                      key='test.txt',
+                                      iso_date='20140625T000000Z')
 
-        self.assertIn('a937f5fbc125d98ac8f04c49e0204ea1526a7b8ca058000a54c192457be05b7d', url)
+        self.assertIn(
+            'a937f5fbc125d98ac8f04c49e0204ea1526a7b8ca058000a54c192457be05b7d',
+            url)
 
     def test_sigv4_presign_optional_params(self):
         self.config = {
@@ -130,11 +156,32 @@ class TestSigV4Presigned(MockServiceWithConfigTestCase):
         )
 
         url = conn.generate_url_sigv4(86400, 'GET', bucket='examplebucket',
-            key='test.txt', version_id=2)
+                                      key='test.txt', version_id=2)
 
         self.assertIn('VersionId=2', url)
         self.assertIn('X-Amz-Security-Token=token', url)
 
+    def test_sigv4_presign_respect_query_auth(self):
+        self.config = {
+            's3': {
+                'use-sigv4': True,
+            }
+        }
+
+        conn = self.connection_class(
+            aws_access_key_id='less',
+            aws_secret_access_key='more',
+            host='s3.amazonaws.com'
+        )
+
+        url_enabled = conn.generate_url(86400, 'GET', bucket='examplebucket',
+                                        key='test.txt', query_auth=True)
+
+        url_disabled = conn.generate_url(86400, 'GET', bucket='examplebucket',
+                                         key='test.txt', query_auth=False)
+        self.assertIn('Signature=', url_enabled)
+        self.assertNotIn('Signature=', url_disabled)
+
     def test_sigv4_presign_headers(self):
         self.config = {
             's3': {
@@ -155,6 +202,26 @@ class TestSigV4Presigned(MockServiceWithConfigTestCase):
         self.assertIn('host', url)
         self.assertIn('x-amz-meta-key', url)
 
+    def test_sigv4_presign_response_headers(self):
+        self.config = {
+            's3': {
+                'use-sigv4': True,
+            }
+        }
+
+        conn = self.connection_class(
+            aws_access_key_id='less',
+            aws_secret_access_key='more',
+            host='s3.amazonaws.com'
+        )
+
+        response_headers = {'response-content-disposition': 'attachment; filename="file.ext"'}
+        url = conn.generate_url_sigv4(86400, 'GET', bucket='examplebucket',
+                                      key='test.txt', response_headers=response_headers)
+
+        self.assertIn('host', url)
+        self.assertIn('response-content-disposition', url)
+
 
 class TestUnicodeCallingFormat(AWSMockServiceTestCase):
     connection_class = S3Connection
